#include <getopt.h>
#include <netlink/msg.h>
#include <netlink/genl/genl.h>
#include <netlink/genl/ctrl.h>

#include "kiosk.h"

static char sstr[MAX_DATA];

void print_usage()
{
	printf(_("Usage:\n"));
	printf(_("-h, --help - show this help\n"));
	printf(_("-u, --user-list-append - append given path to user whitelist\n"));
	printf(_("-U, --user-list-remove - remove given path from user whitelist\n"));
	printf(_("-l, --user-list - show paths in user whitelist\n"));
	printf(_("-m, --set-mode - set kiosk mode (0 = disabled, 1 = enabled)\n"));
	printf(_("-M, --get-mode - get kiosk mode\n"));
}

int print_payload(struct nl_msg *msg, void *arg)
{
	(void) arg; /* unused parameter */
	struct nlattr *attr[KIOSK_MAX_ATTR + 1];

	struct nlmsghdr *hdr = nlmsg_hdr(msg);
	struct genlmsghdr *gehdr = genlmsg_hdr(hdr);

	if (gehdr->cmd != KIOSK_REPLY)
		return NL_SKIP;

	genlmsg_parse(nlmsg_hdr(msg), 0, attr,
			KIOSK_MAX_ATTR, kiosk_attrs_policy);

	if (!attr[KIOSK_DATA])
		return NL_OK;

	char *str = nla_get_string(attr[KIOSK_DATA]);

	if (strlen(str))
		printf("%s\n", str);

	strlcpy(sstr, str, MAX_DATA);

	return NL_OK;
}

int send_msg(struct kiosk_msg *msg, struct nl_sock *sock, int family_id)
{
	int rc = 0;

	struct nl_msg *nlmsg = nlmsg_alloc();
	if (!nlmsg) {
		printf(_("Error allocating message\n"));
		return -1;
	}

	if (!genlmsg_put(nlmsg, 0, 0, family_id, 0, NLM_F_REQUEST, 0, KIOSK_REQUEST)) {
		printf(_("Error putting nl hdr: %d\n"), rc);
		perror(N_("hdr"));
		return rc;
	}

	rc = nla_put_string(nlmsg, KIOSK_DATA, msg->data);
	if (rc < 0) {
		printf(_("Error putting nl string: %d\n"), rc);
		perror(N_("str"));
		return rc;
	}

	rc = nla_put_s16(nlmsg, KIOSK_ACTION, msg->action);
	if (rc < 0) {
		printf(_("Error putting nl action: %d\n"), rc);
		perror(N_("action"));
		return rc;
	}

	rc = nl_send_auto(sock, nlmsg);
	if (rc < 0) {
		printf(_("Error sending message! Error: %d\n"), rc);
		perror(N_("send"));
		return rc;
	}

	nlmsg_free(nlmsg);

	return (rc < 0) ? rc : 0;
}

int recv_msgs(struct nl_sock *sock)
{
	int rc = 0;

	/* we expect an answer */
	rc = nl_recvmsgs_default(sock);

	if (rc < 0) {
		printf(_("Error receiving message! Error: %d\n"), rc);
		perror(N_("recv"));
	}

	return rc;
}

int set_mode(struct kiosk_msg *msg, struct nl_sock *sock, int family_id)
{
	send_msg(msg, sock, family_id);

	if (!strcmp(msg->data, "") && recv_msgs(sock))
		return -1;

	return 0;
}

int get_list(struct kiosk_msg *msg, struct nl_sock *sock, int family_id)
{
	do {
		send_msg(msg, sock, family_id);
		recv_msgs(sock);
		nl_wait_for_ack(sock);
	} while(strlen(sstr));

	return 0;
}

int main (int argc, char **argv)
{
	int c, rc, family;
	struct nl_sock *sock;
	struct kiosk_msg message;

	setlocale(LC_ALL, "");
	textdomain(PACKAGE);

	sock = nl_socket_alloc();
	rc = genl_connect(sock);
	if (rc) {
		printf(_("Error connecting to socket: %d\n"), rc);
		perror(N_("connect"));
		return -1;
	}

	family = genl_ctrl_resolve(sock, "kiosk");
	if (family < 0) {
		printf(_("Error resolving socket family: %d %d\n"), rc, family);
		perror(N_("resolve"));
		return -1;
	}

	memset (&message, 0, sizeof(message));
	message.action = KIOSK_UNKNOWN;

	while(1) {
		static struct option long_options[] = {
			{"help",                no_argument,       0, 'h'},
			{"user-list",           no_argument,       0, 'l'},
			{"set-mode",            required_argument, 0, 'm'},
			{"get-mode",            no_argument,       0, 'M'},
			{"user-list-append",    required_argument, 0, 'u'},
			{"user-list-remove",    required_argument, 0, 'U'},
			{0, 0, 0, 0}
		};

		int option_index = 0;

		c = getopt_long (argc, argv, "hlMm:u:U:",
				long_options, &option_index);

		if (c == -1)
			break;

		switch (c) {
			case '?': /* fallthrough */
			case 'h':
				print_usage();
				break;
			case 'u':
				printf(_("Will append %s to user whitelist\n"), optarg);
				message.action = KIOSK_USERLIST_ADD;
				strlcpy(message.data, optarg, MAX_DATA);
				break;
			case 'U':
				printf(_("Will remove %s from user whitelist\n"), optarg);
				message.action = KIOSK_USERLIST_DEL;
				strlcpy(message.data, optarg, MAX_DATA);
				break;
			case 'l':
				message.action = KIOSK_USER_LIST;
				break;
			case 'm':
				printf(_("Will change mode to %s\n"), optarg);
				strlcpy(message.data, optarg, MAX_DATA);
			case 'M': /* fallthrough */
				message.action = KIOSK_SET_MODE;
				break;
			default:
				return -1;
		}
	}

	int family_id = genl_ctrl_resolve(sock, "kiosk");
	if (family_id < 0) {
		printf(_("Error resolving family: %d\n"), family_id);
		perror(N_("resolve"));
		return -1;
	}

	nl_socket_modify_cb(sock, NL_CB_MSG_IN, NL_CB_CUSTOM, print_payload, NULL);
	nl_socket_disable_seq_check(sock);

	switch(message.action) {
		case KIOSK_USER_LIST: /* fallthrough */
			rc = get_list(&message, sock, family_id);
			break;
		case KIOSK_SET_MODE:
			rc = set_mode(&message, sock, family_id);
			break;
		case KIOSK_USERLIST_ADD: /* fallthrough to USERLIST_DEL */
		case KIOSK_USERLIST_DEL:
			rc = send_msg(&message, sock, family_id);
			break;
	}

	nl_close(sock);

	return rc;
}
